import 'dart:async';
import 'brady_flutter_plugin_platform_interface.dart';

class BradyFlutterPlugin {

  /// startPrinterDiscovery() starts the Bluetooth scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  void startPrinterDiscovery() {
    BradyFlutterPluginPlatform.instance.startPrinterDiscovery();
  }

  /// getPrinters() returns a List of Strings representing the printer names that have been discovered via the discovery scan.
  Future<List<String?>> getPrinters() async {
    return await BradyFlutterPluginPlatform.instance.getPrinters();
  }

  /// connectToPrinter(String) attempts to connect to the printer whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  Future<bool?> connectToPrinter(String? printer) async {
     return await BradyFlutterPluginPlatform.instance.connectToPrinter(printer);
  }

  /// getPrinterDetails() returns a String representing all of the details of the printer the app is currently connected to.
  Future<String?> getPrinterDetails() async {
    return await BradyFlutterPluginPlatform.instance.getPrinterDetails();
  }

  /// getLastConnectedPrinterName() returns a String representing the name of the last printer that the app connected to.
  ///
  /// This is available to allow automatic connection. Since most users own and use only one printer, this can be called immediately after startPrinterDiscovery().
  /// Therefore, when a printer is discovered with the same name as the "last connected printer" you may choose to call the connectToPrinter(String) method to connect to the familiar printer immediately at start up.
  Future<String?> getLastConnectedPrinterName() async {
    return await BradyFlutterPluginPlatform.instance.getLastConnectedPrinterName();
  }

  /// setTemplate(String) takes a base64 string of data representing a template file.
  ///
  /// This is converted to useful information about the template in the back-end code.
  /// This method sets the current template being acted upon when setPlaceholderValue(String, String) or print() is called.
  /// Therefore, calling setTemplate(String) again will override any previous template that has been set.
  void setTemplate(String dataString) {
    return BradyFlutterPluginPlatform.instance.setTemplate(dataString);
  }

  /// setPlaceholderValue(String, String) takes two Strings to set a placeholder with a specified value.
  ///
  /// The first argument must be a String which matches the placeholder's name in Brady Workstation where the template was created.
  /// The second argument must be a String representing the value you want to set to the placeholder.
  void setPlaceholderValue(String placeholder, String value) {
    return BradyFlutterPluginPlatform.instance.setPlaceholderValue(placeholder, value);
  }

  /// print() asynchronously sends a print job of the currently set template to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  Future<bool?> print() async {
    return await BradyFlutterPluginPlatform.instance.print();
  }

  /// feed() asynchronously makes the connected printer print one empty label and returns a Boolean value afterwards with the result of the feed operation(i.e. success or failure).
  Future<bool?> feed() async {
    return await BradyFlutterPluginPlatform.instance.feed();
  }

  /// cut() asynchronously makes the connected printer perform a cut operation (this can be called right after feed() for testing purposes) and returns a Boolean value afterwards with the result of the cut operation (i.e. success or failure).
  Future<bool?> cut() async {
    return await BradyFlutterPluginPlatform.instance.cut();
  }

  /// disconnect() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will also clear the value of "LastConnectedPrinterName" internally.
  Future<bool?> disconnect() async {
    return await BradyFlutterPluginPlatform.instance.disconnect();
  }
}
