import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'brady_flutter_plugin_platform_interface.dart';

/// An implementation of [BradyFlutterPluginPlatform] that uses method channels.
class MethodChannelBradyFlutterPlugin extends BradyFlutterPluginPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('brady_flutter_plugin');

  /// startPrinterDiscovery() starts the Bluetooth scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  @override
  void startPrinterDiscovery() {
    methodChannel.invokeMethod<String>('startPrinterDiscovery');
  }

  /// getPrinters() returns a List of Strings representing the printer names that have been discovered via the discovery scan.
  @override
  Future<List<String?>> getPrinters() async {
    String? printerNames = await methodChannel.invokeMethod<String>('getPrinters');
    List<String> printerNamesList = printerNames!.split(', ');
    printerNamesList.removeLast();
    return printerNamesList;
  }

  /// connectToPrinter(String) attempts to connect to the printer whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  @override
  Future<bool?> connectToPrinter(String? printer) async {
    var sendMap = <String, dynamic> {
      "printerName" : printer
    };
    bool? connected = await methodChannel.invokeMethod<bool>('connectToPrinter', sendMap);
    return connected;
  }

  /// getPrinterDetails() returns a String representing all of the details of the printer the app is currently connected to.
  @override
  Future<String?> getPrinterDetails() async {
    String? details = await methodChannel.invokeMethod<String>('getPrinterDetails');
    return details;
  }

  /// getLastConnectedPrinterName() returns a String representing the name of the last printer that the app connected to.
  ///
  /// This is available to allow automatic connection. Since most users own and use only one printer, this can be called immediately after startPrinterDiscovery().
  /// Therefore, when a printer is discovered with the same name as the "last connected printer" you may choose to call the connectToPrinter(String) method to connect to the familiar printer immediately at start up.
  @override
  Future<String?> getLastConnectedPrinterName() async {
    String? lastConnectedPrinterName = await methodChannel.invokeMethod<String>('getLastConnectedPrinterName');
    return lastConnectedPrinterName;
  }

  /// setTemplate(String) takes a base64 string of data representing a template file.
  ///
  /// This is converted to useful information about the template in the back-end code.
  /// This method sets the current template being acted upon when setPlaceholderValue(String, String) or print() is called.
  /// Therefore, calling setTemplate(String) again will override any previous template that has been set.
  @override
  void setTemplate(String dataString) {
    var sendMap = <String, dynamic> {
      "byteData" : dataString
    };
    methodChannel.invokeMethod('setTemplate', sendMap);
  }

  /// setPlaceholderValue(String, String) takes two Strings to set a placeholder with a specified value.
  ///
  /// The first argument must be a String which matches the placeholder's name in Brady Workstation where the template was created.
  /// The second argument must be a String representing the value you want to set to the placeholder.
  @override
  void setPlaceholderValue(String placeholder, String value) {
    var sendMap = <String, dynamic> {
      "placeholder" : placeholder,
      "value" : value
    };
    methodChannel.invokeMethod('setPlaceholderValue', sendMap);
  }

  /// print() asynchronously sends a print job of the currently set template to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  @override
  Future<bool?> print() async {
    bool? printed = await methodChannel.invokeMethod<bool>('print');
    return printed;
  }

  /// feed() asynchronously makes the connected printer print one empty label and returns a Boolean value afterwards with the result of the feed operation(i.e. success or failure).
  @override
  Future<bool?> feed() async {
    bool? fed = await methodChannel.invokeMethod<bool>('feed');
    return fed;
  }

  /// cut() asynchronously makes the connected printer perform a cut operation (this can be called right after feed() for testing purposes) and returns a Boolean value afterwards with the result of the cut operation (i.e. success or failure).
  @override
  Future<bool?> cut() async {
    bool? cut = await methodChannel.invokeMethod<bool>('cut');
    return cut;
  }

  /// disconnect() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will also clear the value of "LastConnectedPrinterName" internally.
  @override
  Future<bool?> disconnect() async {
    bool? disconnected = await methodChannel.invokeMethod<bool>('disconnect');
    return disconnected;
  }
}
